@@ -1,16 +1,28 @@
-from gevent import monkey; monkey.patch_all()
+from gevent import monkey
+monkey.patch_all()
 from gevent.server import StreamServer
 from gevent.queue import Queue
+
+# Detect if this is Python 2 or 3
+import sys
+_IS_PYTHON_2 = False
+if sys.version_info[ 0 ] < 3:
+    _IS_PYTHON_2 = True
+
+if _IS_PYTHON_2:
+    from urllib2 import urlopen
+else:
+    from urllib.request import urlopen
+
 import sys
 import ssl
 import json
-import urllib2
 import os
 import tempfile
 import socket
 import traceback
 
-from .utils import *
+from .utils import LcApiException
 
 class Firehose( object ):
     '''Listener object to receive data (Events, Detects or Audit) from a limacharlie.io Organization in push mode.'''
@@ -70,7 +82,7 @@ class Firehose( object ):
 
         if self._data_type not in ( 'event', 'detect', 'audit' ):
             raise LcApiException( 'Invalid data type: %s' % self._data_type )
-        
+
         # Setup internal structures.
         self.queue = Queue( maxsize = self._max_buffer )
 
@@ -114,7 +126,7 @@ class Firehose( object ):
                     isStrict = 'false'
                 kwOutputArgs = {
                     'dest_host': effectiveDest,
-                    'is_tls': 'true', 
+                    'is_tls': 'true',
                     'is_strict_tls': isStrict,
                     'is_no_header': 'true',
                 }
@@ -128,9 +140,9 @@ class Firehose( object ):
                     kwOutputArgs[ 'sid' ] = sid
                 if self._is_delete_on_failure:
                     kwOutputArgs[ 'is_delete_on_failure' ] = 'true'
-                self._manager.add_output( self._output_name, 
-                                          'syslog', 
-                                          self._data_type, 
+                self._manager.add_output( self._output_name,
+                                          'syslog',
+                                          self._data_type,
                                           **kwOutputArgs )
                 self._manager._printDebug( 'Registration done.' )
             else:
@@ -140,7 +152,7 @@ class Firehose( object ):
 
     def shutdown( self ):
         '''Stop receiving data and potentially unregister the Output (if created here).'''
-        
+
         if self._name is not None:
             self._manager._printDebug( 'Unregistering.' )
             self._manager.del_output( self._output_name )
@@ -156,7 +168,7 @@ class Firehose( object ):
         self._dropped = 0
 
     def _getPublicIp( self ):
-        return json.load(urllib2.urlopen('http://jsonip.com'))['ip']
+        return json.load( urlopen( 'http://jsonip.com' ) )[ 'ip' ]
 
     def _handleNewClient( self, sock, address ):
         self._manager._printDebug( 'new firehose connection: %s' % ( address, ) )
@@ -167,8 +179,8 @@ class Firehose( object ):
         sock.setsockopt( socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 2 )
 
         try:
-            sock = self._sslCtx.wrap_socket( sock, 
-                                             server_side = True, 
+            sock = self._sslCtx.wrap_socket( sock,
+                                             server_side = True,
                                              do_handshake_on_connect = True,
                                              suppress_ragged_eofs = True )
         except:
@@ -181,35 +193,29 @@ class Firehose( object ):
             try:
                 data = sock.recv( 1024 * 512 )
                 if not data: break
-                if '\n' in data:
-                    chunks = [ c for c in data.split( '\n' ) if c != '' ]
+
+                chunks = [ c for c in data.split( '\n' ) ]
+
+                # This is a pure continuation.
+                if 1 == len( chunks ):
                     curData.append( chunks[ 0 ] )
+                    continue
+
+                # Every chunk is an event boundary.
+                for c in chunks:
+                    curData.append( c )
+                    buff = ''.join( curData )
+                    curData = []
+                    if 0 == len( buff ):
+                        continue
                     try:
                         if self._is_parse:
-                            self.queue.put_nowait( json.loads( ''.join( curData ) ) )
+                            self.queue.put_nowait( json.loads( buff ) )
                         else:
-                            self.queue.put_nowait( ''.join( curData ) )
+                            self.queue.put_nowait( buff )
                     except:
                         self.dropped += 1
-
-                    for c in chunks[ 1 : -1 ]:
-                        try:
-                            if self._is_parse:
-                                self.queue.put_nowait( json.loads( c ) )
-                            else:
-                                self.queue.put_nowait( c )
-                        except:
-                            self.dropped += 1
-                    if 1 < len( chunks ):
-                        if data[ -1 ] == '\n':
-                            if self._is_parse:
-                                self.queue.put_nowait( json.loads( chunks[ -1 ] ) )
-                            else:
-                                self.queue.put_nowait( chunks[ -1 ] )
-                        else:
-                            curData = [ chunks[ -1 ] ]
-                else:
-                    curData.append( data )
+                    buff = None
             except:
                 self._manager._printDebug( 'error decoding data' )
 
@@ -287,7 +293,7 @@ if __name__ == "__main__":
 
     _printToStderr( "Registering..." )
     man = limacharlie.Manager( oid = args.oid, secret_api_key = secretApiKey )
-    fh = limacharlie.Firehose( man, args.listen_interface, args.data_type, 
+    fh = limacharlie.Firehose( man, args.listen_interface, args.data_type,
                                public_dest = args.public_dest,
                                name = args.name,
                                inv_id = args.inv_id,
