@@ -1,17 +1,34 @@
-import urllib2
-import urllib
+# Detect if this is Python 2 or 3
+import sys
+_IS_PYTHON_2 = False
+if sys.version_info[ 0 ] < 3:
+    _IS_PYTHON_2 = True
+
+if _IS_PYTHON_2:
+    from urllib2 import HTTPError
+    from urllib2 import Request as URLRequest
+    from urllib2 import urlopen
+    from urllib import urlencode
+else:
+    from urllib.error import HTTPError
+    from urllib.request import Request as URLRequest
+    from urllib.request import urlopen
+    from urllib.parse import urlencode
+
 import uuid
 import json
 import traceback
 import cmd
-import sys
 import zlib
 import base64
 from functools import wraps
 
 from .Sensor import Sensor
 from .Spout import Spout
-from .utils import *
+from .utils import LcApiException
+from .utils import GET
+from .utils import POST
+from .utils import DELETE
 
 from limacharlie import GLOBAL_OID
 from limacharlie import GLOBAL_UID
@@ -27,7 +44,7 @@ HTTP_UNAUTHORIZED = 401
 class Manager( object ):
     '''General interface to a limacharlie.io Organization.'''
 
-    def __init__( self, oid, secret_api_key, inv_id = None, print_debug_fn = None, is_interactive = False, extra_params = {}, jwt = None, uid = None ):
+    def __init__( self, oid, secret_api_key, inv_id = None, print_debug_fn = None, is_interactive = False, extra_params = {}, jwt = None, uid = None, onRefreshAuth = None ):
         '''Create a session manager for interaction with limacharlie.io, much of the Python API relies on this object.
 
         Args:
@@ -39,6 +56,7 @@ class Manager( object ):
             extra_params (dict): optional key / values passed to interactive spout.
             jwt (str): optionally specify a single JWT to use for authentication.
             uid (str): a limacharlie.io user ID, if present authentication will be based on it instead of organization ID.
+            onRefreshAuth (func): if provided, function is called whenever a JWT would be refreshed using the API key.
         '''
         # If no creds were provided, use the global ones.
         if oid is None:
@@ -64,6 +82,7 @@ class Manager( object ):
                 raise LcApiException( 'Invalid secret API key, should be in UUID format.' )
         self._oid = oid
         self._uid = uid
+        self._onRefreshAuth = onRefreshAuth
         self._secret_api_key = secret_api_key
         self._jwt = jwt
         self._debug = print_debug_fn
@@ -78,7 +97,7 @@ class Manager( object ):
             self._refreshSpout()
 
     def _unwrap( self, data ):
-        return json.loads( zlib.decompress( base64.b64decode( data ), 16 + zlib.MAX_WBITS ) )
+        return json.loads( zlib.decompress( base64.b64decode( data ), 16 + zlib.MAX_WBITS ).decode() )
 
     def _refreshSpout( self ):
         if not self._is_interactive:
@@ -101,19 +120,22 @@ class Manager( object ):
                 authData[ 'uid' ] = self._uid
             else:
                 authData[ 'oid' ] = self._oid
-            request = urllib2.Request( API_TO_JWT_URL,
-                                       urllib.urlencode( authData ) )
+            request = URLRequest( API_TO_JWT_URL,
+                                  urlencode( authData ).encode() )
             request.get_method = lambda: "POST"
-            u = urllib2.urlopen( request )
-            self._jwt = json.loads( u.read() )[ 'jwt' ]
+            u = urlopen( request )
+            self._jwt = json.loads( u.read().decode() )[ 'jwt' ]
             u.close()
         except Exception as e:
             self._jwt = None
             raise LcApiException( 'Failed to get JWT from API key: %s' % e )
 
-    def _restCall( self, url, verb, params, altRoot = None, queryParams = None, rawBody = None, contentType = None ):
+    def _restCall( self, url, verb, params, altRoot = None, queryParams = None, rawBody = None, contentType = None, isNoAuth = False ):
         try:
-            headers = { "Authorization" : "bearer %s" % self._jwt }
+            if not isNoAuth:
+                headers = { "Authorization" : "bearer %s" % self._jwt }
+            else:
+                headers = {}
 
             if altRoot is None:
                 url = '%s/%s/%s' % ( ROOT_URL, API_VERSION, url )
@@ -121,30 +143,30 @@ class Manager( object ):
                 url = '%s/%s' % ( altRoot, url )
 
             if queryParams is not None:
-                url = '%s?%s' % ( url, urllib.urlencode( queryParams ) )
+                url = '%s?%s' % ( url, urlencode( queryParams ) )
 
-            request = urllib2.Request( url,
-                                       rawBody if rawBody is not None else urllib.urlencode( params, doseq = True ),
-                                       headers = headers )
+            request = URLRequest( url,
+                                  rawBody if rawBody is not None else urlencode( params, doseq = True ).encode(),
+                                  headers = headers )
             request.get_method = lambda: verb
             request.add_header( 'User-Agent', 'lc-py-api' )
             if contentType is not None:
                 request.add_header( 'Content-Type', contentType )
-            u = urllib2.urlopen( request )
+            u = urlopen( request )
             try:
                 data = u.read()
                 if 0 != len( data ):
-                    resp = json.loads( data )
+                    resp = json.loads( data.decode() )
                 else:
                     resp = {}
             except ValueError as e:
                 LcApiException( "Failed to decode data from API: %s" % e )
             u.close()
             ret = ( 200, resp )
-        except urllib2.HTTPError as e:
+        except HTTPError as e:
             errorBody = e.read()
             try:
-                ret = ( e.getcode(), json.loads( errorBody ) )
+                ret = ( e.getcode(), json.loads( errorBody.decode() ) )
             except:
                 ret = ( e.getcode(), errorBody )
 
@@ -152,21 +174,27 @@ class Manager( object ):
 
         return ret
 
-    def _apiCall( self, url, verb, params = {}, altRoot = None, queryParams = None, rawBody = None, contentType = None ):
-        if self._jwt is None:
-            self._refreshJWT()
+    def _apiCall( self, url, verb, params = {}, altRoot = None, queryParams = None, rawBody = None, contentType = None, isNoAuth = False ):
+        if not isNoAuth and self._jwt is None:
+            if self._onRefreshAuth is not None:
+                self._onRefreshAuth( self )
+            else:
+                self._refreshJWT()
 
-        code, data = self._restCall( url, verb, params, altRoot = altRoot, queryParams = queryParams, rawBody = rawBody, contentType = contentType )
+        code, data = self._restCall( url, verb, params, altRoot = altRoot, queryParams = queryParams, rawBody = rawBody, contentType = contentType, isNoAuth = isNoAuth )
 
-        if code == HTTP_UNAUTHORIZED:
-            self._refreshJWT()
+        if code == HTTP_UNAUTHORIZED and not isNoAuth:
+            if self._onRefreshAuth is not None:
+                self._onRefreshAuth( self )
+            else:
+                self._refreshJWT()
             code, data = self._restCall( url, verb, params, altRoot = altRoot, queryParams = queryParams, rawBody = rawBody, contentType = contentType )
 
         if 200 != code:
             raise LcApiException( 'Api failure (%s): %s' % ( code, str( data ) ) )
 
         return data
-    
+
     def shutdown( self ):
         '''Shut down any active mechanisms like interactivity.
         '''
@@ -179,10 +207,10 @@ class Manager( object ):
         '''
         if self._is_interactive:
             return
-        
+
         if not self._inv_id:
             raise LcApiException( 'Investigation ID must be set for interactive mode to be enabled.' )
-        
+
         self._is_interactive = True
         self._refreshSpout()
 
@@ -201,7 +229,10 @@ class Manager( object ):
             # First make sure we have an API key or JWT.
             if self._secret_api_key is not None:
                 try:
-                    self._refreshJWT()
+                    if self._onRefreshAuth is not None:
+                        self._onRefreshAuth( self )
+                    else:
+                        self._refreshJWT()
                 except:
                     return False
             elif self._jwt is not None:
@@ -294,7 +325,7 @@ class Manager( object ):
             self._lastSensorListContinuationToken = None
 
         sensors = []
-        resp = self._apiCall( 'sensors/%s' % self._oid, GET, params )
+        resp = self._apiCall( 'sensors/%s' % self._oid, GET, queryParams = params )
         if inv_id is None:
             inv_id = self._inv_id
         for s in resp[ 'sensors' ]:
@@ -343,7 +374,7 @@ class Manager( object ):
         '''
 
         req = { 'name' : name, 'module' : module, 'type' : type }
-        for k, v  in kwargs.iteritems():
+        for k, v in kwargs.items():
             req[ k ] = v
         return self._apiCall( 'outputs/%s' % self._oid, POST, req )
 
@@ -357,36 +388,45 @@ class Manager( object ):
             a list of Sensor IDs matching the hostname expression.
         '''
 
-        req = { 'hostname' : hostname_expr }
-        sensors = []
-        resp = self._apiCall( 'hostnames/%s' % self._oid, GET, req )
-        for s in resp[ 'sid' ]:
-            sensors.append( self.sensor( s, self._inv_id ) )
-        return sensors
+        return self.getSensorsWithHostname( hostname_expr )
 
-    def rules( self ):
+    def rules( self, namespace = None ):
         '''Get the list of all Detection & Response rules for the Organization.
 
+        Args:
+            namespace (str): optional namespace to operator on, defaults to "general".
+
         Returns:
             a list of D&R rules (JSON).
         '''
 
-        resp = self._apiCall( 'rules/%s' % self._oid, GET )
+        req = {}
+        if namespace is not None:
+            req[ 'namespace' ] = namespace
+
+        resp = self._apiCall( 'rules/%s' % self._oid, GET, queryParams = req )
         return resp
 
-    def del_rule( self, name ):
+    def del_rule( self, name, namespace = None ):
         '''Remove a Rule from the Organization.
 
         Args:
             name (str): the name of the Rule to remove.
+            namespace (str): optional namespace to operator on, defaults to "general".
 
         Returns:
             the REST API response (JSON).
         '''
 
-        return self._apiCall( 'rules/%s' % self._oid, DELETE, { 'name' : name } )
+        req = {
+            'name' : name,
+        }
+        if namespace is not None:
+            req[ 'namespace' ] = namespace
 
-    def add_rule( self, name, detection, response, isReplace = False ):
+        return self._apiCall( 'rules/%s' % self._oid, DELETE, req )
+
+    def add_rule( self, name, detection, response, isReplace = False, namespace = None ):
         '''Add a Rule to the Organization.
 
         For detailed explanation and possible Rules parameters
@@ -395,6 +435,7 @@ class Manager( object ):
 
         Args:
             name (str): name to give to the Rule.
+            namespace (str): optional namespace to operator on, defaults to "general".
             isReplace (boolean): if True, replace existing Rule with the same name.
             detection (dict): dictionary representing the detection component of the Rule.
             response (list): list representing the response component of the Rule.
@@ -410,6 +451,9 @@ class Manager( object ):
             'response' : json.dumps( response ),
         }
 
+        if namespace is not None:
+            req[ 'namespace' ] = namespace
+
         return self._apiCall( 'rules/%s' % self._oid, POST, req )
 
     def isInsightEnabled( self ):
@@ -452,7 +496,7 @@ class Manager( object ):
         if cat is not None:
             req[ 'cat' ] = cat
 
-        data = self._apiCall( 'insight/%s/detections' % ( self._oid, ), GET, req )
+        data = self._apiCall( 'insight/%s/detections' % ( self._oid, ), GET, queryParams = req )
         return self._unwrap( data[ 'detects' ] )
 
     def getObjectInformation( self, objType, objName, info, isCaseSensitive = True, isWithWildcards = False ):
@@ -485,7 +529,7 @@ class Manager( object ):
             'per_object' : 'true' if ( isWithWildcards and 'summary' == info ) else 'false',
         }
 
-        data = self._apiCall( 'insight/%s/objects/%s' % ( self._oid, objType ), GET, req )
+        data = self._apiCall( 'insight/%s/objects/%s' % ( self._oid, objType ), GET, queryParams = req )
         return data
 
     def getBatchObjectInformation( self, objects, isCaseSensitive = True ):
@@ -498,7 +542,7 @@ class Manager( object ):
         Returns:
             a dict with keys as time ranges and values are maps of object types to object name lists.
         '''
-        for objType, objNames in objects.iteritems():
+        for objType, objNames in objects.items():
             objects[ objType ] = list( objNames )
         req = {
             'objects' : json.dumps( objects ),
@@ -540,7 +584,7 @@ class Manager( object ):
         Returns:
             List of (sid, hostname).
         '''
-        data = self._apiCall( 'hostnames/%s' % ( self._oid, ), GET, {
+        data = self._apiCall( 'hostnames/%s' % ( self._oid, ), GET, queryParams = {
             'hostname' : hostnamePrefix,
         } )
         return data.get( 'sid', None )
@@ -556,8 +600,8 @@ class Manager( object ):
             Dict with general success, or data from Replicant if isSynchronous.
         '''
         data = self._apiCall( 'replicant/%s/%s' % ( self._oid, replicantName ), POST, {
-            'request_data' : base64.b64encode( json.dumps( data ) ),
-            'is_async' : not isSynchronous,
+            'request_data' : base64.b64encode( json.dumps( data ).encode() ),
+            'is_async' : isSynchronous,
         } )
         return data
 
@@ -587,7 +631,7 @@ class Manager( object ):
         Returns:
             Dictionary of resource types to URLs.
         '''
-        data = self._apiCall( 'orgs/%s/url' % ( self._oid, ), GET )
+        data = self._apiCall( 'orgs/%s/url' % ( self._oid, ), GET, isNoAuth = True )
         return data.get( 'url', None )
 
     def getIngestionKeys( self ):
@@ -623,14 +667,121 @@ class Manager( object ):
         } )
         return data
 
+    def setOrgQuota( self, quota ):
+        '''Set a new sensor quota for the organization.
+
+        Args:
+            quota (int): the new quota value.
+        '''
+        data = self._apiCall( 'orgs/%s/quota' % ( self._oid, ), POST, {
+            'quota' : int( quota ),
+        } )
+        return data
+
+    def getSubscriptions( self ):
+        '''Get the list of resources the organization is subscribed to.
+
+        '''
+        data = self._apiCall( 'orgs/%s/resources' % ( self._oid, ), GET, {} )
+        return data.get( 'resources', None )
+
+    def subscribeToResource( self, name ):
+        '''Subscribe the organization to the specific resource.
+
+        Args:
+            name (str): name of the resource like lookup/test-res.
+        '''
+        resCat, resName = name.split( '/' )
+        data = self._apiCall( 'orgs/%s/resources' % ( self._oid, ), POST, {
+            'res_cat': resCat,
+            'res_name' : resName,
+        } )
+        return data
+
+    def unsubscribeFromResource( self, name ):
+        '''Unsubscribe the organization from the specific resource.
+
+        Args:
+            name (str): name of the resource like lookup/test-res.
+        '''
+        resCat, resName = name.split( '/' )
+        data = self._apiCall( 'orgs/%s/resources' % ( self._oid, ), DELETE, {
+            'res_cat': resCat,
+            'res_name' : resName,
+        } )
+        return data
+
+    def getUsers( self ):
+        '''Get the list of users in the organization.
+
+        '''
+        data = self._apiCall( 'orgs/%s/users' % ( self._oid, ), GET, {} )
+        return data.get( 'users', None )
+
+    def addUser( self, email ):
+        '''Add a user to an organization.
+
+        Args:
+            email (str): email of the user to add.
+        '''
+        data = self._apiCall( 'orgs/%s/users' % ( self._oid, ), POST, {
+            'email' : email,
+        } )
+        return data
+
+    def removeUser( self, email ):
+        '''Remove user from an organization.
+
+        Args:
+            email (str): email of the user to remove.
+        '''
+        data = self._apiCall( 'orgs/%s/users' % ( self._oid, ), DELETE, {
+            'email' : email,
+        } )
+        return data
+
+    def getUserPermissions( self ):
+        '''Get the list of users and their permissions.
+
+        '''
+        data = self._apiCall( 'orgs/%s/users/permissions' % ( self._oid, ), GET, {} )
+        return data.get( 'user_permissions', None )
+
+    def addUserPermission( self, email, permission ):
+        '''Add a user to an organization.
+
+        Args:
+            email (str): email of the user to add.
+            permission (str): permission to add to the user.
+        '''
+        data = self._apiCall( 'orgs/%s/users/permissions' % ( self._oid, ), POST, {
+            'email' : email,
+            'perm' : permission,
+        } )
+        return data
+
+    def removeUserPermission( self, email, permission ):
+        '''Remove user from an organization.
+
+        Args:
+            email (str): email of the user to remove.
+            permission (str): permission to remove from the user.
+        '''
+        data = self._apiCall( 'orgs/%s/users/permissions' % ( self._oid, ), DELETE, {
+            'email' : email,
+            'perm' : permission,
+        } )
+        return data
+
 def _eprint( msg ):
-    print >> sys.stderr, msg
+    sys.stderr.write( msg )
+    sys.stderr.write( "\n" )
 
 def _report_errors( func ):
     @wraps( func )
     def silenceit( *args, **kwargs ):
         try:
-            return func( *args,**kwargs )
+            return func( *args, **kwargs )
         except:
             _eprint( traceback.format_exc() )
             return None
